; -*- mode:     CL -*- ----------------------------------------------------- ;
; File:         zebu-aux.l
; Description:  Functions and structures common to compiler and driver
; Author:       Joachim H. Laubsch
; Created:      11-Oct-90
; Modified:     Fri Mar 12 14:43:35 1993 (Joachim H. Laubsch)
; Language:     CL
; Package:      ZEBU
; Status:       Experimental (Do Not Distribute) 
; RCS $Header: $
;
; (c) Copyright 1990, Hewlett-Packard Company
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; Revisions:
; RCS $Log: $
;  7-Apr-92 (Joachim H. Laubsch)
;  many efficiency improvements throughout based on using Lucid's monitor
;  facility.
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(in-package "ZEBU")
(provide "zebu-aux")

#-LUCID (proclaim '(special *load-source-pathname-types* 
                            *load-binary-pathname-types*))
#+(or MCL Allegro)
(setq *load-source-pathname-types* '("lisp" NIL)
      *load-binary-pathname-types* '("fasl"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                  Functions common to runtime and compiler
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defun list->vector (l)
  (let* ((len (length l))
	 (v (make-sequence 'vector len)))
    (declare (vector v))
    (dotimes (i len v)
      (setf (aref v i) (pop l)))))

(defun identifierp (x)
  (and (symbolp x) (not (null x))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;          Lexical analysis (regex) Run/Compile time data structures
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;; Declare the global variables for storing the paren index list.
;;;
(defvar *regex-groups* (make-array 10))
(defvar *regex-groupings* 0)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                     External representation
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defvar *zb-rules*)                     ; alist of rule-names and zb-rule structs

(defstruct zb-rule
  -name
  -productions)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                   Internal Representation of Productions
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;; About the internal representation of productions:
;;;  production-index:  (0 .. Number of productions - 1)
;;;  lhs:               a g-symbol
;;;  rhs:               a list of g-symbols
;;;  production-length: the length of rhs

(defstruct (production (:conc-name nil))
  lhs
  rhs
  production-index
  production-length)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                   check the first form of a grammar file
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; this applies to a .zb as well as a .tab file

(proclaim '(special *compiler-grammar* *identifier-continue-chars*
	    *identifier-start-chars*))

(defun check-grammar-options (options filename compiling
				      &aux g-name)
  ;; check the list of options for plausibility
  ;; on package conflict, Nil is returned to catch point: read-grammar-options
  ;; we must then read the options again with *package* set correctly
  (flet ((wrng-make-grammar-arglist (key)
	   (error "~S is not a defined keyword for make-grammar." key)))
    (format t "~%Reading grammar from ~A~%" filename)
    (do ((gg options (cddr gg))) ((null gg))
      (let ((key (car gg)) (val (cadr gg)))
	(if (keywordp key)
	    (case key
	      (:NAME    (setq g-name val))
	      (:PACKAGE
	       (let ((p (find-package val)))
		 (if p
		     (progn
		       (use-package "ZEBU" p)
		       (unless (eq *package* p)
			 (setq *package* p)
			 (throw 'read-grammar-options nil)))
		   (error
		    "Package ~s should be defined before ~:[loading~;compiling~] ~S"
		    val compiling filename))))
	      (:GRAMMAR (let ((g (find-grammar val)))
			  (if g
			      (setq *compiler-grammar* g)
			    (warn "Grammar ~S is not loaded" val))))
	      (:IDENTIFIER-CONTINUE-CHARS
	       (setf *identifier-continue-chars* val))
	      (:IDENTIFIER-START-CHARS
	       (setf *identifier-start-chars* val))
	      ((:STRING-DELIMITER :SYMBOL-DELIMITER :FILE :DOMAIN
		:LEX-CATS :WHITE-SPACE :DOMAIN-FILE
		:INTERN-IDENTIFIER))
	      (t (wrng-make-grammar-arglist key)))
	  (wrng-make-grammar-arglist key))))
    (unless g-name
      (setq g-name (pathname-name filename)
	    options (list* ':NAME g-name options)))
    (when compiling
      (setq options (list* ':FILE (namestring filename) options)))
    options))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                      The Root of the Domain Hierarchy
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defstruct (kb-domain (:constructor nil)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;          Global Variables (shared by runtime system and compiler)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

#-LUCID 
(defvar *KEYWORD-PACKAGE* (find-package "KEYWORD"))

(defvar *generate-domain* t
  "If true while Zebu compiling a grammar, generate the hierarchy
otherwise the domain-hierarchy is written by the user.")

(defvar *ZEBU-PACKAGE* (find-package "ZEBU"))
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                                   Version
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(defvar *zebu-version* "2.8.5")

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                             End of zebu-aux.l
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
